# Any class including this module will be able to process jobs. A
# developer can include this module in their class and configure
# it with `stomp_options`. e.g.
#
# class MyWorker
#   include StompJob::Worker
#   stomp_options queue_name: 'my_queue', other_opts: '...'

#   def perform(args)
#     # do some work
#   end
# end

module StompJob
  module Worker

    def self.included(base_worker)
      base_worker.extend ClassMethods
      Workers.register( base_worker )
    end

    module ClassMethods
      attr_accessor :queue_name, :stomp_connection

      # stomp_options controls parameters for the worker and the
      # queue. All supported options are:
      #
      # queue_name (required) - The name of the queue to use on
      #                         the queue broker. Will be created
      #                         if it doesn't already exist.
      # stomp_connection      - The STOMP connection to use for
      #                         the worker. An instance of
      #                         StompJob::Connection. By default,
      #                         all workers will use the same
      #                         STOMP connection.
      def stomp_options(opts={})
        opts = opts.with_indifferent_access if opts.respond_to?(:with_indifferent_access)
        @queue_name = opts[:queue_name]
        @stomp_connection = opts[:stomp_connection]
      end

      # Enqueue a payload for a worker
      def enqueue(*args)
        stomp_connection.publish(queue_name, *args)
      end

      # Make the current connection listen for messages that
      # should be handled by this worker.
      # Typically invoked by the Consumer.
      def listen!
        stomp_connection.subscribe(self)
      end

      def stomp_connection
        @stomp_connection ||= Connection.default_connection
      end
    end

  end
end
